=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2012-01-25 17:41:05 +0000
+++ bzrlib/builtins.py	2012-01-28 00:41:30 +0000
@@ -83,10 +83,11 @@
     )
 
 
-def _get_branch_location(control_dir):
+def _get_branch_location(control_dir, possible_transports=None):
     """Return location of branch for this control dir."""
     try:
-        this_branch = control_dir.open_branch()
+        this_branch = control_dir.open_branch(
+            possible_transports=possible_transports)
         # This may be a heavy checkout, where we want the master branch
         master_location = this_branch.get_bound_location()
         if master_location is not None:
@@ -101,7 +102,34 @@
             return control_dir.root_transport.base
 
 
-def lookup_new_sibling_branch(control_dir, location):
+def _is_colocated(control_dir, possible_transports=None):
+    """Check if the branch in control_dir is colocated.
+
+    :param control_dir: Control directory
+    :return: Boolean indicating whether 
+    """
+    # This path is meant to be relative to the existing branch
+    this_url = _get_branch_location(control_dir,
+        possible_transports=possible_transports)
+    # Perhaps the target control dir supports colocated branches?
+    try:
+        root = controldir.ControlDir.open(this_url,
+            possible_transports=possible_transports)
+    except errors.NotBranchError:
+        return (False, this_url)
+    else:
+        try:
+            wt = control_dir.open_workingtree()
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            return (False, this_url)
+        else:
+            return (
+                root._format.colocated_branches and
+                control_dir.control_url == root.control_url,
+                this_url)
+
+
+def lookup_new_sibling_branch(control_dir, location, possible_transports=None):
     """Lookup the location for a new sibling branch.
 
     :param control_dir: Control directory relative to which to look up
@@ -111,16 +139,7 @@
     """
     location = directory_service.directories.dereference(location)
     if '/' not in location and '\\' not in location:
-        # This path is meant to be relative to the existing branch
-        this_url = _get_branch_location(control_dir)
-        # Perhaps the target control dir supports colocated branches?
-        try:
-            root = controldir.ControlDir.open(this_url,
-                possible_transports=[control_dir.user_transport])
-        except errors.NotBranchError:
-            colocated = False
-        else:
-            colocated = root._format.colocated_branches
+        (colocated, this_url) = _is_colocated(control_dir, possible_transports)
 
         if colocated:
             return urlutils.join_segment_parameters(this_url,
@@ -130,7 +149,7 @@
     return location
 
 
-def lookup_sibling_branch(control_dir, location):
+def lookup_sibling_branch(control_dir, location, possible_transports=None):
     """Lookup sibling branch.
     
     :param control_dir: Control directory relative to which to lookup the
@@ -140,7 +159,8 @@
     """
     try:
         # Perhaps it's a colocated branch?
-        return control_dir.open_branch(location)
+        return control_dir.open_branch(location, 
+            possible_transports=possible_transports)
     except (errors.NotBranchError, errors.NoColocatedBranchSupport):
         try:
             return Branch.open(location)
@@ -6201,14 +6221,17 @@
         from bzrlib import switch
         tree_location = directory
         revision = _get_one_revision('switch', revision)
-        control_dir = controldir.ControlDir.open_containing(tree_location)[0]
+        possible_transports = []
+        control_dir = controldir.ControlDir.open_containing(tree_location,
+            possible_transports=possible_transports)[0]
         if to_location is None:
             if revision is None:
                 raise errors.BzrCommandError(gettext('You must supply either a'
                                              ' revision or a location'))
             to_location = tree_location
         try:
-            branch = control_dir.open_branch()
+            branch = control_dir.open_branch(
+                possible_transports=possible_transports)
             had_explicit_nick = branch.get_config().has_explicit_nickname()
         except errors.NotBranchError:
             branch = None
@@ -6217,9 +6240,10 @@
             if branch is None:
                 raise errors.BzrCommandError(
                     gettext('cannot create branch without source branch'))
-            to_location = lookup_new_sibling_branch(control_dir, to_location)
+            to_location = lookup_new_sibling_branch(control_dir, to_location,
+                 possible_transports=possible_transports)
             to_branch = branch.bzrdir.sprout(to_location,
-                 possible_transports=[branch.bzrdir.root_transport],
+                 possible_transports=possible_transports,
                  source_branch=branch).open_branch()
         else:
             to_branch = lookup_sibling_branch(control_dir, to_location)

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2012-01-27 20:46:27 +0000
+++ bzrlib/bzrdir.py	2012-01-28 02:05:52 +0000
@@ -810,10 +810,51 @@
     present within a BzrDir.
     """
 
+    def _get_branch_path(self, name):
+        """Obtain the branch path to use.
+
+        This uses the API specified branch name first, and then falls back to
+        the branch name specified in the URL. If neither of those is specified,
+        it uses the default branch.
+
+        :param name: Optional branch name to use
+        :return: Relative path to branch
+        """
+        if name == "":
+            return 'branch'
+        return urlutils.join('branches', name.encode("utf-8"))
+
+    def _read_branch_list(self):
+        """Read the branch list.
+
+        :return: List of utf-8 encoded branch names.
+        """
+        try:
+            f = self.control_transport.get('branch-list')
+        except errors.NoSuchFile:
+            return []
+
+        ret = []
+        try:
+            for name in f:
+                ret.append(name.rstrip("\n"))
+        finally:
+            f.close()
+        return ret
+
+    def _write_branch_list(self, branches):
+        """Write out the branch list.
+
+        :param branches: List of utf-8 branch names to write
+        """
+        self.transport.put_bytes('branch-list',
+            "".join([name+"\n" for name in branches]))
+
     def __init__(self, _transport, _format):
         super(BzrDirMeta1, self).__init__(_transport, _format)
-        self.control_files = lockable_files.LockableFiles(self.control_transport,
-            self._format._lock_file_name, self._format._lock_class)
+        self.control_files = lockable_files.LockableFiles(
+            self.control_transport, self._format._lock_file_name,
+            self._format._lock_class)
 
     def can_convert_format(self):
         """See BzrDir.can_convert_format()."""
@@ -832,9 +873,19 @@
         """See BzrDir.create_branch."""
         if name is None:
             name = self._get_selected_branch()
+        path = self._get_branch_path(name)
         if name != "":
-            raise errors.NoColocatedBranchSupport(self)
-        self.transport.delete_tree('branch')
+            self.control_files.lock_write()
+            try:
+                branches = self._read_branch_list()
+                try:
+                    branches.remove(name.encode("utf-8"))
+                except ValueError:
+                    raise errors.NotBranchError(name)
+                self._write_branch_list(branches)
+            finally:
+                self.control_files.unlock()
+        self.transport.delete_tree(path)
 
     def create_repository(self, shared=False):
         """See BzrDir.create_repository."""
@@ -896,21 +947,41 @@
         """See BzrDir.get_branch_transport()."""
         if name is None:
             name = self._get_selected_branch()
-        if name != "":
-            raise errors.NoColocatedBranchSupport(self)
+        path = self._get_branch_path(name)
         # XXX: this shouldn't implicitly create the directory if it's just
         # promising to get a transport -- mbp 20090727
         if branch_format is None:
-            return self.transport.clone('branch')
+            return self.transport.clone(path)
         try:
             branch_format.get_format_string()
         except NotImplementedError:
             raise errors.IncompatibleFormat(branch_format, self._format)
+        if name != "":
+            branches = self._read_branch_list()
+            utf8_name = name.encode("utf-8")
+            if not utf8_name in branches:
+                self.control_files.lock_write()
+                try:
+                    branches = self._read_branch_list()
+                    dirname = urlutils.dirname(utf8_name)
+                    if dirname != "" and dirname in branches:
+                        raise errors.ParentBranchExists(name)
+                    child_branches = [
+                        b.startswith(utf8_name+"/") for b in branches]
+                    if any(child_branches):
+                        raise errors.AlreadyBranchError(name)
+                    branches.append(utf8_name)
+                    self._write_branch_list(branches)
+                finally:
+                    self.control_files.unlock()
+        branch_transport = self.transport.clone(path)
+        mode = self._get_mkdir_mode()
+        branch_transport.create_prefix(mode=mode)
         try:
-            self.transport.mkdir('branch', mode=self._get_mkdir_mode())
+            self.transport.mkdir(path, mode=mode)
         except errors.FileExists:
             pass
-        return self.transport.clone('branch')
+        return self.transport.clone(path)
 
     def get_repository_transport(self, repository_format):
         """See BzrDir.get_repository_transport()."""
@@ -940,6 +1011,19 @@
             pass
         return self.transport.clone('checkout')
 
+    def get_branches(self):
+        """See ControlDir.get_branches."""
+        ret = {}
+        try:
+            ret[""] = self.open_branch(name="")
+        except (errors.NotBranchError, errors.NoRepositoryPresent):
+            pass
+
+        for name in self._read_branch_list():
+            ret[name] = self.open_branch(name=name.decode('utf-8'))
+
+        return ret
+
     def has_workingtree(self):
         """Tell if this bzrdir contains a working tree.
 
@@ -1013,124 +1097,6 @@
         return config.TransportConfig(self.transport, 'control.conf')
 
 
-class BzrDirMeta1Colo(BzrDirMeta1):
-    """BzrDirMeta1 with support for colocated branches.
-
-    This format is experimental, and will eventually be merged back into
-    BzrDirMeta1.
-    """
-
-    def _get_branch_path(self, name):
-        """Obtain the branch path to use.
-
-        This uses the API specified branch name first, and then falls back to
-        the branch name specified in the URL. If neither of those is specified,
-        it uses the default branch.
-
-        :param name: Optional branch name to use
-        :return: Relative path to branch
-        """
-        if name == "":
-            return 'branch'
-        return urlutils.join('branches', name.encode("utf-8"))
-
-    def _read_branch_list(self):
-        """Read the branch list.
-
-        :return: List of utf-8 encoded branch names.
-        """
-        try:
-            f = self.control_transport.get('branch-list')
-        except errors.NoSuchFile:
-            return []
-
-        ret = []
-        try:
-            for name in f:
-                ret.append(name.rstrip("\n"))
-        finally:
-            f.close()
-        return ret
-
-    def _write_branch_list(self, branches):
-        """Write out the branch list.
-
-        :param branches: List of utf-8 branch names to write
-        """
-        self.transport.put_bytes('branch-list',
-            "".join([name+"\n" for name in branches]))
-
-    def destroy_branch(self, name=None):
-        """See BzrDir.create_branch."""
-        if name is None:
-            name = self._get_selected_branch()
-        path = self._get_branch_path(name)
-        if name != "":
-            self.control_files.lock_write()
-            try:
-                branches = self._read_branch_list()
-                try:
-                    branches.remove(name.encode("utf-8"))
-                except ValueError:
-                    raise errors.NotBranchError(name)
-                self._write_branch_list(branches)
-            finally:
-                self.control_files.unlock()
-        self.transport.delete_tree(path)
-
-    def get_branches(self):
-        """See ControlDir.get_branches."""
-        ret = {}
-        try:
-            ret[""] = self.open_branch(name="")
-        except (errors.NotBranchError, errors.NoRepositoryPresent):
-            pass
-
-        for name in self._read_branch_list():
-            ret[name] = self.open_branch(name=name.decode('utf-8'))
-
-        return ret
-
-    def get_branch_transport(self, branch_format, name=None):
-        """See BzrDir.get_branch_transport()."""
-        if name is None:
-            name = self._get_selected_branch()
-        path = self._get_branch_path(name)
-        # XXX: this shouldn't implicitly create the directory if it's just
-        # promising to get a transport -- mbp 20090727
-        if branch_format is None:
-            return self.transport.clone(path)
-        try:
-            branch_format.get_format_string()
-        except NotImplementedError:
-            raise errors.IncompatibleFormat(branch_format, self._format)
-        if name != "":
-            branches = self._read_branch_list()
-            utf8_name = name.encode("utf-8")
-            if not utf8_name in branches:
-                self.control_files.lock_write()
-                try:
-                    branches = self._read_branch_list()
-                    dirname = urlutils.dirname(utf8_name)
-                    if dirname != "" and dirname in branches:
-                        raise errors.ParentBranchExists(name)
-                    child_branches = [
-                        b.startswith(utf8_name+"/") for b in branches]
-                    if any(child_branches):
-                        raise errors.AlreadyBranchError(name)
-                    branches.append(utf8_name)
-                    self._write_branch_list(branches)
-                finally:
-                    self.control_files.unlock()
-        branch_transport = self.transport.clone(path)
-        branch_transport.create_prefix()
-        try:
-            self.transport.mkdir('.', mode=self._get_mkdir_mode())
-        except errors.FileExists:
-            pass
-        return branch_transport
-
-
 class BzrFormat(object):
     """Base class for all formats of things living in metadirs.
 
@@ -1579,7 +1545,7 @@
 
     fixed_components = False
 
-    colocated_branches = False
+    colocated_branches = True
 
     def __init__(self):
         BzrDirFormat.__init__(self)
@@ -1712,7 +1678,7 @@
             return ConvertMetaToColo(format)
         if (type(self) is BzrDirMetaFormat1Colo and
             type(format) is BzrDirMetaFormat1):
-            return ConvertMetaRemoveColo(format)
+            return ConvertMetaToColo(format)
         if not isinstance(self, format.__class__):
             # converting away from metadir is not implemented
             raise NotImplementedError(self.get_converter)
@@ -1814,7 +1780,7 @@
         # problems.
         format = BzrDirMetaFormat1Colo()
         self._supply_sub_formats_to(format)
-        return BzrDirMeta1Colo(transport, format)
+        return BzrDirMeta1(transport, format)
 
 
 BzrProber.formats.register(BzrDirMetaFormat1Colo.get_format_string(),
@@ -1916,12 +1882,12 @@
         return BzrDir.open_from_transport(to_convert.root_transport)
 
 
-class ConvertMetaRemoveColo(controldir.Converter):
-    """Remove colocated branch support from a bzrdir."""
+class ConvertMetaToColo(controldir.Converter):
+    """Convert a 'development-colo' bzrdir to a '2a' bzrdir."""
 
     def __init__(self, target_format):
-        """Create a converter.that downgrades a colocated branch metadir
-        to a regular metadir.
+        """Create a converter that converts a 'development-colo' metadir
+        to a '2a' metadir.
 
         :param target_format: The final metadir format that is desired.
         """
@@ -1929,14 +1895,6 @@
 
     def convert(self, to_convert, pb):
         """See Converter.convert()."""
-        to_convert.control_files.lock_write()
-        try:
-            branches = to_convert.list_branches()
-            if len(branches) > 1:
-                raise errors.BzrError("remove all but a single "
-                    "colocated branch when downgrading")
-        finally:
-            to_convert.control_files.unlock()
         to_convert.transport.put_bytes('branch-format',
             self.target_format.as_string())
         return BzrDir.open_from_transport(to_convert.root_transport)

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2012-01-27 13:33:53 +0000
+++ bzrlib/remote.py	2012-01-28 00:56:56 +0000
@@ -59,7 +59,7 @@
 from bzrlib.repository import RepositoryWriteLockResult, _LazyListJoin
 from bzrlib.serializer import format_registry as serializer_format_registry
 from bzrlib.trace import mutter, note, warning, log_exception_quietly
-from bzrlib.versionedfile import ChunkedContentFactory, FulltextContentFactory
+from bzrlib.versionedfile import FulltextContentFactory
 
 
 _DEFAULT_SEARCH_DEPTH = 100
@@ -113,6 +113,8 @@
 
     supports_workingtrees = False
 
+    colocated_branches = False
+
     def __init__(self):
         _mod_bzrdir.BzrDirMetaFormat1.__init__(self)
         # XXX: It's a bit ugly that the network name is here, because we'd
@@ -627,6 +629,8 @@
                       append_revisions_only=None):
         if name is None:
             name = self._get_selected_branch()
+        if name != "":
+            raise errors.NoColocatedBranchSupport(self)
         # as per meta1 formats - just delegate to the format object which may
         # be parameterised.
         real_branch = self._format.get_branch_format().initialize(self,
@@ -651,9 +655,13 @@
 
     def destroy_branch(self, name=None):
         """See BzrDir.destroy_branch"""
+        if name is None:
+            name = self._get_selected_branch()
+        if name != "":
+            raise errors.NoColocatedBranchSupport(self)
         path = self._path_for_remote_call(self._client)
         try:
-            if name is not None:
+            if name != "":
                 args = (name, )
             else:
                 args = ()
@@ -699,13 +707,18 @@
 
     def set_branch_reference(self, target_branch, name=None):
         """See BzrDir.set_branch_reference()."""
+        if name is None:
+            name = self._get_selected_branch()
+        if name != "":
+            raise errors.NoColocatedBranchSupport(self)
         self._ensure_real()
         return self._real_bzrdir.set_branch_reference(target_branch, name=name)
 
     def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
-        if name is not None:
-            # XXX JRV20100304: Support opening colocated branches
+        if name is None:
+            name = self._get_selected_branch()
+        if name != "":
             raise errors.NoColocatedBranchSupport(self)
         response = self._get_branch_reference()
         if response[0] == 'ref':
@@ -765,6 +778,10 @@
 
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False, possible_transports=None):
+        if name is None:
+            name = self._get_selected_branch()
+        if name != "":
+            raise errors.NoColocatedBranchSupport(self)
         if unsupported:
             raise NotImplementedError('unsupported flag support not implemented yet.')
         if self._next_open_branch_result is not None:
@@ -773,8 +790,6 @@
             self._next_open_branch_result = None
             return result
         response = self._get_branch_reference()
-        if name is None:
-            name = self._get_selected_branch()
         return self._open_branch(name, response[0], response[1],
             possible_transports=possible_transports,
             ignore_fallbacks=ignore_fallbacks)
@@ -3122,10 +3137,10 @@
         if isinstance(a_bzrdir, RemoteBzrDir):
             a_bzrdir._ensure_real()
             result = self._custom_format.initialize(a_bzrdir._real_bzrdir,
-                name, append_revisions_only=append_revisions_only)
+                name=name, append_revisions_only=append_revisions_only)
         else:
             # We assume the bzrdir is parameterised; it may not be.
-            result = self._custom_format.initialize(a_bzrdir, name,
+            result = self._custom_format.initialize(a_bzrdir, name=name,
                 append_revisions_only=append_revisions_only)
         if (isinstance(a_bzrdir, RemoteBzrDir) and
             not isinstance(result, RemoteBranch)):
@@ -3322,6 +3337,7 @@
         # will try to assign to self.tags, which is a property in this subclass.
         # And the parent's __init__ doesn't do much anyway.
         self.bzrdir = remote_bzrdir
+        self.name = name
         if _client is not None:
             self._client = _client
         else:

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2012-01-18 19:14:21 +0000
+++ bzrlib/switch.py	2012-01-26 15:31:02 +0000
@@ -45,7 +45,11 @@
         source_repository = control_dir.open_branch().repository
     except errors.NotBranchError:
         source_repository = to_branch.repository
-    _set_branch_location(control_dir, to_branch, force)
+    to_branch.lock_read()
+    try:
+        _set_branch_location(control_dir, to_branch, force)
+    finally:
+        to_branch.unlock()
     tree = control_dir.open_workingtree()
     _update(tree, source_repository, quiet, revision_id)
     _run_post_switch_hooks(control_dir, to_branch, force, revision_id)
@@ -109,15 +113,19 @@
         else:
             # If this is a standalone tree and the new branch
             # is derived from this one, create a lightweight checkout.
-            graph = b.repository.get_graph(to_branch.repository)
-            if (b.bzrdir._format.colocated_branches and
-                 (force or graph.is_ancestor(b.last_revision(),
-                    to_branch.last_revision()))):
-                b.bzrdir.destroy_branch()
-                b.bzrdir.set_branch_reference(to_branch, name="")
-            else:
-                raise errors.BzrCommandError(gettext('Cannot switch a branch, '
-                    'only a checkout.'))
+            b.lock_read()
+            try:
+                graph = b.repository.get_graph(to_branch.repository)
+                if (b.bzrdir._format.colocated_branches and
+                     (force or graph.is_ancestor(b.last_revision(),
+                        to_branch.last_revision()))):
+                    b.bzrdir.destroy_branch()
+                    b.bzrdir.set_branch_reference(to_branch, name="")
+                else:
+                    raise errors.BzrCommandError(gettext('Cannot switch a branch, '
+                        'only a checkout.'))
+            finally:
+                b.unlock()
 
 
 def _any_local_commits(this_branch, possible_transports):

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2012-01-25 17:41:05 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2012-01-28 00:41:30 +0000
@@ -155,14 +155,15 @@
         #  => new branch will be created, but switch fails and the current
         #     branch is unmodified
         self.example_branch('a')
-        self.make_branch_and_tree('current')
+        tree = self.make_branch_and_tree('current')
+        c1 = tree.commit('some diverged change')
         self.run_bzr_error(['Cannot switch a branch, only a checkout'],
             'branch --switch ../a ../b', working_dir='current')
         a = branch.Branch.open('a')
         b = branch.Branch.open('b')
         self.assertEqual(a.last_revision(), b.last_revision())
         work = branch.Branch.open('current')
-        self.assertEqual(work.last_revision(), _mod_revision.NULL_REVISION)
+        self.assertEqual(work.last_revision(), c1)
 
     def test_branch_into_empty_dir(self):
         t = self.example_branch('source')

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2012-01-19 16:51:39 +0000
+++ bzrlib/tests/blackbox/test_info.py	2012-01-26 15:36:39 +0000
@@ -99,6 +99,9 @@
         branch: Branch format 5
     repository: Knit repository format 1
 
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified
@@ -130,6 +133,9 @@
         branch: Branch format 5
     repository: Knit repository format 1
 
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified
@@ -184,6 +190,9 @@
         branch: Branch format 5
     repository: Knit repository format 1
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -230,6 +239,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -273,6 +285,9 @@
         branch: Branch format 5
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -316,6 +331,9 @@
         branch: Branch format 5
     repository: Knit repository format 1
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -361,6 +379,9 @@
         branch: Branch format 5
     repository: Knit repository format 1
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -400,6 +421,9 @@
         branch: Branch format 5
     repository: %s
 
+Control directory:
+         1 branches
+
 Branch is out of date: missing 1 revision.
 
 In the working tree:
@@ -439,6 +463,9 @@
         branch: Branch format 5
     repository: %s
 
+Control directory:
+         1 branches
+
 Branch is out of date: missing 1 revision.
 
 In the working tree:
@@ -478,6 +505,9 @@
         branch: Branch format 5
     repository: Knit repository format 1
 
+Control directory:
+         1 branches
+
 Working tree is out of date: missing 1 revision.
 
 In the working tree:
@@ -517,6 +547,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 Branch history:
          0 revisions
 
@@ -545,6 +578,9 @@
        control: Meta directory format 1
     repository: %s
 
+Control directory:
+         0 branches
+
 Repository:
          0 revisions
 """ % ('repo', format.repository_format.get_format_description(),
@@ -567,6 +603,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 Branch history:
          0 revisions
 
@@ -611,6 +650,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -649,6 +691,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 Branch is out of date: missing 1 revision.
 
 In the working tree:
@@ -688,6 +733,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -730,6 +778,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 Working tree is out of date: missing 1 revision.
 
 In the working tree:
@@ -769,6 +820,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 Branch history:
          2 revisions
          0 days old
@@ -794,6 +848,9 @@
        control: Meta directory format 1
     repository: %s
 
+Control directory:
+         0 branches
+
 Repository:
          2 revisions
 """ % (format.repository_format.get_format_description(),
@@ -817,6 +874,9 @@
        control: Meta directory format 1
     repository: %s
 
+Control directory:
+         0 branches
+
 Create working tree for new branches inside the repository.
 
 Repository:
@@ -845,6 +905,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified
@@ -885,6 +948,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -926,6 +992,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified
@@ -965,6 +1034,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          1 unchanged
          0 modified
@@ -1000,6 +1072,9 @@
        control: Meta directory format 1
     repository: %s
 
+Control directory:
+         0 branches
+
 Create working tree for new branches inside the repository.
 
 Repository:
@@ -1026,6 +1101,9 @@
        control: Meta directory format 1
     repository: %s
 
+Control directory:
+         0 branches
+
 Create working tree for new branches inside the repository.
 
 Repository:
@@ -1051,6 +1129,9 @@
         branch: %s
     repository: %s
 
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified
@@ -1088,6 +1169,9 @@
        control: Meta directory format 1
     repository: %s
 
+Control directory:
+         0 branches
+
 Create working tree for new branches inside the repository.
 
 Repository:
@@ -1227,6 +1311,9 @@
         branch: %s
     repository: %s
 %s
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified
@@ -1403,6 +1490,9 @@
         branch: Branch format 7
     repository: Repository format 2a - rich roots, group compression and chk inventories
 
+Control directory:
+         1 branches
+
 In the working tree:
          0 unchanged
          0 modified

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2012-01-04 17:12:42 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2012-01-28 00:39:01 +0000
@@ -318,11 +318,17 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             return
+        if repo.bzrdir._format.colocated_branches:
+            raise tests.TestNotApplicable(
+                "control dir does not support colocated branches")
         self.assertEquals(0, len(repo.bzrdir.list_branches()))
+        if not self.bzrdir_format.colocated_branches:
+            raise tests.TestNotApplicable("control dir format does not support "
+                "colocated branches")
         try:
             child_branch1 = self.branch_format.initialize(repo.bzrdir, 
                 name='branch1')
-        except (errors.UninitializableFormat, errors.NoColocatedBranchSupport):
+        except errors.UninitializableFormat:
             # branch references are not default init'able and
             # not all bzrdirs support colocated branches.
             return

=== modified file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2012-01-27 14:36:00 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2012-01-28 00:41:30 +0000
@@ -685,10 +685,9 @@
 
     def test_get_branches(self):
         repo = self.make_repository('branch-1')
-        try:
-            target_branch = repo.bzrdir.create_branch(name='foo')
-        except errors.NoColocatedBranchSupport:
+        if not repo.bzrdir._format.colocated_branches:
             raise TestNotApplicable('Format does not support colocation')
+        target_branch = repo.bzrdir.create_branch(name='foo')
         repo.bzrdir.set_branch_reference(target_branch)
         self.assertEqual(set(["", 'foo']),
                          set(repo.bzrdir.get_branches().keys()))

=== modified file 'bzrlib/tests/per_controldir_colo/__init__.py'
--- bzrlib/tests/per_controldir_colo/__init__.py	2010-08-21 16:06:24 +0000
+++ bzrlib/tests/per_controldir_colo/__init__.py	2012-01-28 00:56:56 +0000
@@ -27,17 +27,23 @@
 from bzrlib.tests import (
     default_transport,
     multiply_tests,
+    test_server,
     )
 from bzrlib.tests.per_controldir import (
     TestCaseWithControlDir,
     make_scenarios,
     )
+from bzrlib.transport import memory
 
 
 def load_tests(standard_tests, module, loader):
     colo_supported_formats = []
     colo_unsupported_formats = []
+    # This will always add scenarios using the smart server.
+    from bzrlib.remote import RemoteBzrDirFormat
     for format in ControlDirFormat.known_formats():
+        if isinstance(format, RemoteBzrDirFormat):
+            continue
         if format.colocated_branches:
             colo_supported_formats.append(format)
         else:
@@ -46,6 +52,22 @@
         colo_supported_formats)
     unsupported_scenarios = make_scenarios(default_transport, None, None,
         colo_unsupported_formats)
+    # test the remote server behaviour when backed with a MemoryTransport
+    # Once for the current version
+    unsupported_scenarios.extend(make_scenarios(
+        memory.MemoryServer,
+        test_server.SmartTCPServer_for_testing,
+        test_server.ReadonlySmartTCPServer_for_testing,
+        [(RemoteBzrDirFormat())],
+        name_suffix='-default'))
+    # And once with < 1.6 - the 'v2' protocol.
+    unsupported_scenarios.extend(make_scenarios(
+        memory.MemoryServer,
+        test_server.SmartTCPServer_for_testing_v2_only,
+        test_server.ReadonlySmartTCPServer_for_testing_v2_only,
+        [(RemoteBzrDirFormat())],
+        name_suffix='-v2'))
+
     result = loader.suiteClass()
     supported_tests = loader.loadTestsFromModuleNames([
         'bzrlib.tests.per_controldir_colo.test_supported'])

=== modified file 'bzrlib/tests/per_controldir_colo/test_unsupported.py'
--- bzrlib/tests/per_controldir_colo/test_unsupported.py	2012-01-18 17:37:38 +0000
+++ bzrlib/tests/per_controldir_colo/test_unsupported.py	2012-01-28 00:56:56 +0000
@@ -40,9 +40,10 @@
             raise tests.TestNotApplicable('Control dir format not supported')
         t = self.get_transport()
         try:
-            made_control = self.bzrdir_format.initialize(t.base)
+            made_control = self.make_bzrdir('.', format=self.bzrdir_format)
         except errors.UninitializableFormat:
             raise tests.TestNotApplicable('Control dir format not initializable')
+        self.assertEquals(made_control._format, self.bzrdir_format)
         made_repo = made_control.create_repository()
         return made_control
 

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2012-01-27 15:47:12 +0000
+++ bzrlib/tests/test_bzrdir.py	2012-01-28 00:41:30 +0000
@@ -1447,8 +1447,8 @@
         tree.bzrdir.create_branch(name="another-colocated-branch")
         converter = tree.bzrdir._format.get_converter(
             bzrdir.BzrDirMetaFormat1())
-        self.assertRaises(errors.BzrError, converter.convert, tree.bzrdir,
-            None)
+        result = converter.convert(tree.bzrdir, bzrdir.BzrDirMetaFormat1())
+        self.assertIsInstance(result._format, bzrdir.BzrDirMetaFormat1)
 
     def test_nested(self):
         tree = self.make_branch_and_tree('.', format='development-colo')

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2012-01-06 22:06:36 +0000
+++ bzrlib/tests/test_remote.py	2012-01-28 01:43:42 +0000
@@ -589,18 +589,6 @@
         a_bzrdir.destroy_branch()
         self.assertFinished(client)
 
-    def test_destroy_named(self):
-        transport = self.get_transport('quack')
-        referenced = self.make_branch('referenced')
-        client = FakeClient(transport.base)
-        client.add_expected_call(
-            'BzrDir.destroy_branch', ('quack/', "foo"),
-            'success', ('ok',)),
-        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
-            _client=client)
-        a_bzrdir.destroy_branch("foo")
-        self.assertFinished(client)
-
 
 class TestBzrDirHasWorkingTree(TestRemote):
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2012-01-20 13:15:15 +0000
+++ bzrlib/transport/__init__.py	2012-01-28 02:05:52 +0000
@@ -364,7 +364,7 @@
         """
         raise NotImplementedError(self.clone)
 
-    def create_prefix(self):
+    def create_prefix(self, mode=None):
         """Create all the directories leading down to self.base."""
         cur_transport = self
         needed = [cur_transport]
@@ -376,7 +376,7 @@
                     "Failed to create path prefix for %s."
                     % cur_transport.base)
             try:
-                new_transport.mkdir('.')
+                new_transport.mkdir('.', mode=mode)
             except errors.NoSuchFile:
                 needed.append(new_transport)
                 cur_transport = new_transport
@@ -387,9 +387,9 @@
         # Now we only need to create child directories
         while needed:
             cur_transport = needed.pop()
-            cur_transport.ensure_base()
+            cur_transport.ensure_base(mode=mode)
 
-    def ensure_base(self):
+    def ensure_base(self, mode=None):
         """Ensure that the directory this transport references exists.
 
         This will create a directory if it doesn't exist.
@@ -399,7 +399,7 @@
         # than permission". We attempt to create the directory, and just
         # suppress FileExists and PermissionDenied (for Windows) exceptions.
         try:
-            self.mkdir('.')
+            self.mkdir('.', mode=mode)
         except (errors.FileExists, errors.PermissionDenied):
             return False
         else:

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2012-01-27 21:40:55 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2012-01-28 00:41:30 +0000
@@ -20,6 +20,9 @@
 
 .. New commands, options, etc that users may wish to try out.
 
+* Support for colocated branches is now available in the default
+  format ("2a"). (Jelmer Vernooij)
+
 Improvements
 ************
 

